@@ -1,10 +1,10 @@
 /**********************************************************************
  *
- minires 1.00         stub synchronous resolver for Cygwin
+ minires 1.01         stub synchronous resolver for Cygwin
  Pierre A. Humblet
- October 2004
+ December 2006
 
- Copyright (c) 2001, 2002, 2003, 2004 Pierre A. Humblet
+ Copyright (c) 2001, 2002, 2003, 2004, 2006 Pierre A. Humblet
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
@@ -219,11 +219,24 @@ static int open_sock(struct sockaddr_in *CliAddr, int debug)
 {
   int fd;
   
+  DPRINTF(debug, "opening UDP socket\n");
+
   /* Create a datagram socket */
   if ((fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
-    DPRINTF(debug, "socket: %s\n", strerror(errno));
+    DPRINTF(debug, "socket(UDP): %s\n", strerror(errno));
     return -1;
   }
+  /* Set non-blocking */
+  if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0)  {
+    DPRINTF(debug, "fcntl: %s\n", strerror(errno));
+    return -1;
+  }
+  /* Set close on exec flag */
+  if (fcntl(fd, F_SETFD, 1) == -1) {
+    DPRINTF(debug, "fcntl: %s\n", strerror(errno));
+    return -1;
+  }
+
   CliAddr->sin_family = AF_INET;
   CliAddr->sin_addr.s_addr = htonl(INADDR_ANY);
   CliAddr->sin_port = htons(0);
@@ -266,7 +279,10 @@ int res_ninit(res_state statp)
   statp->use_os = 1;            /* use os_query if available and allowed by get_resolv */
   statp->mypid = -1;
   statp->sockfd = -1;
-
+  /* Use the pid and the ppid for random seed, from the point of view of an outsider.
+     Mix the upper and lower bits as they are not used equally */
+  i = getpid();
+  statp->id = (ushort) (getppid() ^ (i << 8) ^ (i >> 8));
   for (i = 0; i < DIM(statp->dnsrch); i++)  statp->dnsrch[i] = 0;
 
   /* Get search list from LOCALDOMAIN */
@@ -361,7 +377,7 @@ static int get_tcp(struct sockaddr_in *CliAddr,
 
   /* Create a tcp socket */
   if ((fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
-    DPRINTF(debug, "socket: %s\n", strerror(errno));
+    DPRINTF(debug, "socket(TCP): %s\n", strerror(errno));
     return -1;
   }
 
@@ -406,6 +422,7 @@ int res_nsend( res_state statp, const unsigned char * MsgPtr,
   /* Current server, shared by all tasks */
   volatile static unsigned int SServ = 0XFFFFFFFF;
   int tcp;
+  const int debug = statp->options & RES_DEBUG;
 
   fd_set fdset_read; 
   int rslt, addrLen, transNum, wServ;
@@ -424,25 +441,15 @@ int res_nsend( res_state statp, const unsigned char * MsgPtr,
 
   /* Open a socket for this process */
   if (statp->sockfd == -1) {
-    /* Create a socket and bind it (to any port) */
-    statp->sockfd = open_sock(& mySockAddr, 
-			      statp->options & RES_INIT);
-    DPRINTF(statp->options & RES_DEBUG, "open sockfd %d: %s\n", 
-	    statp->sockfd, (statp->sockfd > 0)?"OK":strerror(errno));
+    /* Create a non-blocking, close on exec socket and bind it (to any port) */
+    statp->sockfd = open_sock(& mySockAddr, debug);
     if (statp->sockfd  < 0 ) {
       statp->res_h_errno = NETDB_INTERNAL;
       return -1;
     }
-    /* Set close on exec flag */
-    if (fcntl(statp->sockfd, F_SETFD, 1) == -1) {
-      DPRINTF(statp->options & RES_DEBUG, "fcntl: %s\n", 
-	      strerror(errno));
-      statp->res_h_errno = NETDB_INTERNAL;
-      return -1;
-    }
     statp->mypid = getpid();
     if (SServ == 0XFFFFFFFF) /* Pseudo random */
-      SServ =  statp->mypid % statp->nscount;
+      SServ =  statp->id % statp->nscount;
   }
 
   transNum = 0;  
@@ -450,11 +457,31 @@ int res_nsend( res_state statp, const unsigned char * MsgPtr,
     if ((wServ = SServ + 1) >= statp->nscount) 
       wServ = 0;
     SServ = wServ;
+
+    /* There exists attacks on DNS where many wrong answers with guessed id's and
+       spoofed source address and port are generated at about the time when the
+       program is tricked into resolving a name.
+       This routine runs through the retry loop for each incorrect answer.
+       It is thus extremely likely that such attacks will cause a TRY_AGAIN return,
+       probably causing the calling program to retry after a delay.
+       
+       Note that valid late or duplicate answers to a previous questions also cause
+       a retry, although this is minimized by flushing the socket before sending the
+       new question.
+    */
+
+    /* Flush duplicate or late answers */
+    while ((rslt = recvfrom( statp->sockfd, AnsPtr, AnsLength, 0, NULL, NULL)) >= 0) {
+      DPRINTF(debug, "Flushed %d bytes\n", rslt);
+    }
+    DPRINTF(debug && (errno != EWOULDBLOCK), 
+	    "Unexpected errno for flushing recvfrom: %s", strerror(errno)); 
+
     /* Send the message */
     rslt = sendto(statp->sockfd, MsgPtr, MsgLength, 0, 
 		  (struct sockaddr *) &statp->nsaddr_list[wServ],
 		  sizeof(struct sockaddr_in));
-    DPRINTF(statp->options & RES_DEBUG, "sendto: server %08x sockfd %d %s\n",
+    DPRINTF(debug, "sendto: server %08x sockfd %d %s\n",
 	    statp->nsaddr_list[wServ].sin_addr.s_addr, 
 	    statp->sockfd, (rslt == MsgLength)?"OK":strerror(errno));
     if (rslt != MsgLength) {
@@ -475,39 +502,52 @@ int res_nsend( res_state statp, const unsigned char * MsgPtr,
       continue;
     }
     else if ((rslt != 1) || (FD_ISSET(statp->sockfd, &fdset_read) == 0)) {
-      DPRINTF(statp->options & RES_DEBUG, "select: %s\n", strerror(errno));
+      DPRINTF(debug, "select: %s\n", strerror(errno));
       statp->res_h_errno = NETDB_INTERNAL;
       return -1;
     }
 
     addrLen = sizeof(dnsSockAddr);
-    rslt = recvfrom( statp->sockfd, AnsPtr, AnsLength, 0, 
-		     (struct sockaddr *) & dnsSockAddr, & addrLen);
+    rslt = recvfrom(statp->sockfd, AnsPtr, AnsLength, 0, 
+		    (struct sockaddr *) & dnsSockAddr, & addrLen);
     if (rslt <= 0) {
-      DPRINTF(statp->options & RES_DEBUG, "recvfrom: %s\n", 
-	      strerror(errno));
+      DPRINTF(debug, "recvfrom: %s\n", strerror(errno));
       statp->res_h_errno = NETDB_INTERNAL;
       return -1;
     }
+    DPRINTF(debug, "recvfrom: %d bytes from %08x\n", rslt, dnsSockAddr.sin_addr.s_addr);
     /* 
        Prepare to retry with tcp 
     */
     for (tcp = 0; tcp < 2; tcp++) {
-      /* Check if this is the message we expected */
-      if ((*MsgPtr == *AnsPtr)     /* Ids match */
-	  && (*(MsgPtr + 1) == *(AnsPtr + 1))
-/* We have stopped checking this because the question may not be present on error,
-   in particular when the name in the question is not a valid name.
-   Simply check that the header is present. */
+      /* Check if this is the expected message from the expected server */
+      if ((memcmp(& dnsSockAddr, & statp->nsaddr_list[wServ],
+		  (char *) & dnsSockAddr.sin_zero[0] - (char *) & dnsSockAddr) == 0)
 	  && (rslt >= HFIXEDSZ)
-/*        && (rslt >= MsgLength )
-	  && (memcmp(MsgPtr + HFIXEDSZ, AnsPtr + HFIXEDSZ, MsgLength - HFIXEDSZ) == 0) */
-	  && ((AnsPtr[2] & QR) != 0)) {
-
-	DPRINTF(statp->options & RES_DEBUG, 
-		"answer %u from %08x. Error %d. Count %d.\n",
-		rslt, dnsSockAddr.sin_addr.s_addr, 
-		AnsPtr[3] & ERR_MASK, AnsPtr[6]*256 + AnsPtr[7]);
+	  && (*MsgPtr == *AnsPtr)     /* Ids match */
+	  && (*(MsgPtr + 1) == *(AnsPtr + 1))
+	  && ((AnsPtr[2] & QR) != 0)
+	  && (AnsPtr[4] == 0)
+	  /* We check the question if present.
+	     Some servers don't return it on error, in particular
+	     when the name in the question is not valid. */
+	  && (((AnsPtr[5] == 0)
+	       && ((AnsPtr[3] & ERR_MASK) != NOERROR))
+	      || ((AnsPtr[5] == 1)
+		  && (rslt >= MsgLength)
+		  && (memcmp(MsgPtr + HFIXEDSZ, AnsPtr + HFIXEDSZ, MsgLength - HFIXEDSZ) == 0)))) {
+	if ((AnsPtr[3] & ERR_MASK) == NOERROR) {
+	  if ((AnsPtr[2] & TC) && (tcp == 0) && !(statp->options & RES_IGNTC)) { 
+	    /* Truncated. Try TCP */
+	    rslt = get_tcp(&statp->nsaddr_list[wServ], MsgPtr, MsgLength,
+			   AnsPtr, AnsLength, statp->options & RES_DEBUG);
+	    continue /* Tcp loop */; 
+	  }
+	  else if ((AnsPtr[6] | AnsPtr[7])!= 0)
+	    return rslt;
+	  else
+	    statp->res_h_errno = NO_DATA;
+	}
 #if 0
  NETDB_INTERNAL -1 /* see errno */
  NETDB_SUCCESS   0 /* no problem */
@@ -517,37 +557,29 @@ int res_nsend( res_state statp, const unsigned char * MsgPtr,
  NO_RECOVERY     3 /* Non recoverable errors, FORMERR, REFUSED, NOTIMP */
  NO_DATA         4 /* Valid name, no data record of requested type */
 #endif
-	if ((AnsPtr[3] & ERR_MASK) == NOERROR) {
-	  if ((AnsPtr[2] & TC) && !(statp->options & RES_IGNTC)) { /* Truncated. Try TCP */
-	    rslt = get_tcp(&statp->nsaddr_list[wServ], MsgPtr, MsgLength,
-			   AnsPtr, AnsLength, statp->options & RES_DEBUG);
-	    continue; 
-	  }
-	  else if ((AnsPtr[6] | AnsPtr[7])!= 0) 
-	    return rslt;
-	  else 
-	    statp->res_h_errno = NO_DATA;
-	}
 	else {
+	  switch (AnsPtr[3] & ERR_MASK) {
 	  /* return HOST_NOT_FOUND even for non-authoritative answers */
-	  if ((AnsPtr[3] & ERR_MASK) == NXDOMAIN) 
+	  case NXDOMAIN:
+	  case FORMERR:
 	    statp->res_h_errno = HOST_NOT_FOUND;
-	  else if ((AnsPtr[3] & ERR_MASK) == SERVFAIL) 
+	    break;
+	  case SERVFAIL: 
 	    statp->res_h_errno = TRY_AGAIN;
-	  else 
+	    break;
+	  default:
 	    statp->res_h_errno = NO_RECOVERY;
+	  }
 	}
 	return -1;
       }
       else {
-	DPRINTF(statp->options & RES_DEBUG, "unexpected answer %u from %x to query to %x\n",
-		rslt, dnsSockAddr.sin_addr.s_addr, 
-		statp->nsaddr_list[wServ].sin_addr.s_addr);
+	DPRINTF(debug, "unexpected answer\n");
 	break;
       }
     } /* TCP */
   }
-  DPRINTF(statp->options & RES_DEBUG, "too many retries\n");
+  DPRINTF(debug, "too many retries\n");
   statp->res_h_errno = TRY_AGAIN;
   return -1;
 }
@@ -573,7 +605,8 @@ int res_nmkquery (res_state statp,
 		  const unsigned char * newrr, unsigned char * buf, int buflen)
 {
   int i, len;
-  short id;
+  const char * ptr;
+  unsigned int id4;
 
   if (op == QUERY) {
     /* Write the name and verify buffer length */
@@ -584,9 +617,9 @@ int res_nmkquery (res_state statp,
       statp->res_h_errno = NETDB_INTERNAL;
       return -1;
     }
+
     /* Fill the header */
-    id = statp->id;
-    PUTSHORT(id, buf);
+    PUTSHORT(statp->id, buf);
     PUTSHORT(RD, buf); 
     PUTSHORT(1, buf); /* Number of questions */
     for (i = 0; i < 3; i++)
@@ -596,7 +629,19 @@ int res_nmkquery (res_state statp,
     buf += len;
     PUTSHORT(qtype, buf);
     PUTSHORT(qclass, buf);
-    return len + 16; /* packet size */
+
+    /* Update id. The current query adds entropy to the next query id */
+    for (id4 = qtype, i = 0, ptr = dnameptr; *ptr; ptr++, i += 3) 
+      id4 ^= *ptr << (i & 0xF);
+    i = 1 + statp->id % 15; /* Between 1 and 16 */
+    /* id dependent rotation, also brings MSW to LSW */
+    id4 = (id4 << i) ^ (id4 >> (16 - i)) ^ (id4 >> (32 - i));
+    if ((short) id4)
+      statp->id ^= (short) id4;
+    else 
+      statp->id++; /* Force change */
+
+    return len + (HFIXEDSZ + QFIXEDSZ); /* packet size */
   }
   else { /* Not implemented */
     errno = ENOSYS;
