@@ -1,10 +1,10 @@
 /**********************************************************************
  *
- minires 0.98         stub synchronous resolver for Cygwin
+ minires 1.01         stub synchronous resolver for Cygwin
  Pierre A. Humblet
- # January 2004
+ December 2006
 
- Copyright (c) 2001, 2002, 2003, 2004 Pierre A. Humblet
+ Copyright (c) 2001, 2002, 2003, 2004, 2006 Pierre A. Humblet
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
@@ -71,7 +71,7 @@ static u_char * write_record(unsigned char * ptr, PDNS_RECORD rr, unsigned char
   ptr += 2; /* Placeholder for RDLENGTH */
 
   /* The default case uses an undocumented feature of the Windows
-     resolver for types greater than 16. 
+     resolver for many types greater than 16. 
      The DNS_RECORD Data contains the record in wire format. */
 
   switch(rr->wType) {
@@ -128,10 +128,13 @@ static u_char * write_record(unsigned char * ptr, PDNS_RECORD rr, unsigned char
     }
     break;
   case DNS_TYPE_MINFO:
+  case DNS_TYPE_RP:
     PUTDOMAIN(rr->Data.MINFO.pNameMailbox, ptr);
     PUTDOMAIN(rr->Data.MINFO.pNameErrorsMailbox, ptr);
     break;
   case DNS_TYPE_MX:
+  case DNS_TYPE_AFSDB:
+  case DNS_TYPE_RT:
     if (ptr + 2 > EndPtr)
       ptr += 2;
     else
@@ -139,7 +142,9 @@ static u_char * write_record(unsigned char * ptr, PDNS_RECORD rr, unsigned char
     PUTDOMAIN(rr->Data.MX.pNameExchange, ptr);
     break;
   case DNS_TYPE_HINFO:
-  case DNS_TYPE_TEXT: 
+  case DNS_TYPE_ISDN:
+  case DNS_TYPE_TEXT:
+  case DNS_TYPE_X25:
   {
     unsigned int i, len;
     for (i = 0; i < rr->Data.TXT.dwStringCount; i++) {
@@ -154,6 +159,16 @@ static u_char * write_record(unsigned char * ptr, PDNS_RECORD rr, unsigned char
     }
     break;
   }
+  case DNS_TYPE_SRV:
+    if (ptr + 6 > EndPtr)
+      ptr += 6;
+    else {
+      PUTSHORT(rr->Data.SRV.wPriority, ptr);
+      PUTSHORT(rr->Data.SRV.wWeight, ptr);
+      PUTSHORT(rr->Data.SRV.wPort, ptr);
+    }
+    PUTDOMAIN(rr->Data.SRV.pNameTarget, ptr);
+    break;
   default:
   {
     unsigned int len = rr->wDataLength;
@@ -250,6 +265,9 @@ static int cygwin_query(res_state statp, const char * DomName, int Class, int Ty
       errno = EINVAL;
       statp->res_h_errno = NETDB_INTERNAL;;
       break;
+    case ERROR_TIMEOUT:
+      statp->res_h_errno = TRY_AGAIN;
+      break;
     case DNS_ERROR_RCODE_NAME_ERROR:
       statp->res_h_errno = HOST_NOT_FOUND;
       break;
